@@ -2,6 +2,7 @@ from typing import Literal
 from typing import Optional
 
 import numpy as np
+from jax import lax
 from jax import nn
 from jax import numpy as jnp
 from jaxtyping import Array
@@ -115,9 +116,14 @@ def multi_head_attention(
             x, x.shape[:-1] + (num_heads, head_dim)
         )  # [..., heads, head_dim]
 
-    Q = jnp.moveaxis(split_heads(Q), -2, 1)  # [batch, heads, seq_q, head_dim]
-    K = jnp.moveaxis(split_heads(K), -2, 1)  # [batch, heads, seq_k, head_dim]
-    V = jnp.moveaxis(split_heads(V), -2, 1)  # [batch, heads, seq_k, head_dim]
+    # Q = jnp.moveaxis(split_heads(Q), -2, 1)  # [batch, heads, seq_q, head_dim]
+    # K = jnp.moveaxis(split_heads(K), -2, 1)  # [batch, heads, seq_k, head_dim]
+    # V = jnp.moveaxis(split_heads(V), -2, 1)  # [batch, heads, seq_k, head_dim]
+
+    if implementation == "cudnn":
+        Q = lax.convert_element_type(Q, jnp.bfloat16)  # TODO make this flexible
+        K = lax.convert_element_type(K, jnp.bfloat16)  # TODO make this flexible
+        V = lax.convert_element_type(V, jnp.bfloat16)  # TODO make this flexible
 
     attn_output = nn.dot_product_attention(
         Q, K, V, mask=masks, is_causal=is_causal, implementation=implementation
